import re

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver

WILDCARD_PATTERN = re.compile(r"(\S+[.][*][.]*)+")


class BaseAttributeSolver(BaseSolver):
    operator = ''

    def __init__(self, resource_types, attribute, value):
        super().__init__(SolverType.ATTRIBUTE)
        self.resource_types = resource_types
        self.attribute = attribute
        self.value = value

    def run(self, graph_connector):
        all_vertices_resource_types = [data for _, data in graph_connector.nodes(data=True) if
                                       self.resource_type_pred(data)]
        passed_vertices = [data for data in all_vertices_resource_types if self.get_operation(vertex=data)]
        failed_vertices = [resource for resource in all_vertices_resource_types if resource not in passed_vertices]
        return passed_vertices, failed_vertices

    def get_operation(self, vertex):
        if not re.match(WILDCARD_PATTERN, self.attribute):
            return self.resource_type_pred(vertex) and self._get_operation(vertex=vertex, attribute=self.attribute)
        attribute_pattern = self.get_attribute_pattern(self.attribute)
        return self.resource_type_pred(vertex) and any(self._get_operation(vertex=vertex, attribute=attr) for attr in vertex if re.match(attribute_pattern, attr))

    def _get_operation(self, vertex, attribute):
        raise NotImplementedError

    def resource_type_pred(self, v):
        return len(self.resource_types) == 0 or v.get('resource_type') in self.resource_types

    @staticmethod
    def get_attribute_pattern(attribute):
        split_by_dots = attribute.split(".")
        pattern_parts = []
        for i, attr_part in enumerate(split_by_dots):
            if attr_part == "*":
                pattern_parts.append(r"[\d+]")
            else:
                pattern_parts.append(f"({attr_part})")
        pattern = "[.]".join(pattern_parts)
        p = re.compile(pattern)
        return p



from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class DMSReplicationInstancePubliclyAccessible(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "DMS replication instance should not be publicly accessible"
        id = "CKV_AWS_89"
        supported_resources = ['aws_dms_replication_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'publicly_accessible'

    def get_forbidden_values(self):
        return [True]


check = DMSReplicationInstancePubliclyAccessible()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class AKSUsesDiskEncryptionSet(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure that AKS uses disk encryption set"
        id = "CKV_AZURE_117"
        supported_resources = ['azurerm_kubernetes_cluster']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return "disk_encryption_set_id"

    def get_forbidden_values(self):
        return ['']


check = AKSUsesDiskEncryptionSet()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class ALBDropHttpHeaders(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that ALB drops HTTP headers"
        id = "CKV_AWS_131"
        supported_resources = ['aws_lb', 'aws_alb']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get('load_balancer_type') == 'network':
            return CheckResult.UNKNOWN
        return super().scan_resource_conf(conf)

    def get_inspected_key(self):
        return 'drop_invalid_header_fields'


check = ALBDropHttpHeaders()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class RDSEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure all data stored in the RDS is securely encrypted at rest"
        id = "CKV_AWS_16"
        supported_resources = ['AWS::RDS::DBInstance']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)
    
    def scan_resource_conf(self, conf):
        # If DB is Aurora then Encryption is set in other resource
        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-storageencrypted
        if 'Properties' in conf.keys():
            if 'Engine' in conf['Properties'].keys():
                if 'aurora' in conf['Properties']['Engine']:
                    return CheckResult.UNKNOWN
        # Database is not Aurora; Use base class implementation
        return super().scan_resource_conf(conf)

    def get_inspected_key(self):
        return 'Properties/StorageEncrypted'


check = RDSEncryption()



from networkx.classes.digraph import DiGraph

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.terraform.checks_infra.solvers.connections_solvers.complex_connection_solver import ComplexConnectionSolver
from checkov.terraform.graph_builder.graph_components.attribute_names import CustomAttributes


class OrConnectionSolver(ComplexConnectionSolver):
    operator = 'or'

    def __init__(self, queries, operator):
        super().__init__(queries, operator)

    def run(self, graph_connector: DiGraph):
        passed, failed = [], []
        for sub_query in self.queries:
            if sub_query.solver_type in [SolverType.ATTRIBUTE, SolverType.CONNECTION]:
                sub_passed, sub_failed = sub_query.run(graph_connector)
                passed.extend(sub_passed)
                failed.extend(sub_failed)
                failed = [f for f in failed if f[CustomAttributes.ID] not in [p[CustomAttributes.ID] for p in passed]]
        return self.filter_results(passed, failed)



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GKEEnsureIntegrityMonitoring(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Integrity Monitoring for Shielded GKE Nodes is Enabled"
        check_id = "CKV_GCP_72"
        supported_resources = ['google_container_cluster', 'google_container_node_pool']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'node_config' in conf.keys():
            node = conf["node_config"][0]
            if isinstance(node, dict) and 'shielded_instance_config' in node.keys():
                monitor = node["shielded_instance_config"][0]
                if monitor["enable_integrity_monitoring"] == [True]:
                    return CheckResult.PASSED
                else:
                    return CheckResult.FAILED
            else:
                # as default is true this is a pass
                return CheckResult.PASSED
        else:
            # no config is valid it could be in the the node_pool
            return CheckResult.UNKNOWN


check = GKEEnsureIntegrityMonitoring()



class Edge:
    def __init__(self, origin, dest, label):
        self.origin = origin
        self.dest = dest
        self.label = label

    def __str__(self):
        return f'[{self.origin} -({self.label})-> {self.dest}]'



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class ELBCrossZoneEnable(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that ELB is cross-zone-load-balancing enabled"
        id = "CKV_AWS_138"
        supported_resources = ['aws_elb']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return 'cross_zone_load_balancing'


check = ELBCrossZoneEnable()



import logging
import json
import logging
import os

from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.runners.base_runner import BaseRunner
from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.registry import resource_registry
from checkov.terraform.context_parsers.registry import parser_registry
# Allow the evaluation of empty variables
from checkov.terraform.plan_parser import parse_tf_plan


class Runner(BaseRunner):
    check_type = "terraform_plan"

    def __init__(self):
        self.tf_definitions = {}
        self.definitions_context = {}
        self.template_lines = {}

    block_type_registries = {
        'resource': resource_registry,
    }

    def run(self, root_folder=None, external_checks_dir=None, files=None, runner_filter=RunnerFilter(),
            collect_skip_comments=True):
        report = Report(self.check_type)
        self.tf_definitions = {}
        parsing_errors = {}
        if external_checks_dir:
            for directory in external_checks_dir:
                resource_registry.load_external_checks(directory, runner_filter)

        if root_folder:
            files = [] if not files else files
            for root, d_names, f_names in os.walk(root_folder):
                for file in f_names:
                    file_ending = os.path.splitext(file)[1]
                    if file_ending == '.json':
                        try:
                            with open(f'{root}/{file}') as f:
                                content = json.load(f)
                            if isinstance(content, dict) and content.get('terraform_version'):
                                files.append(os.path.join(root, file))
                        except Exception as e:
                            logging.debug(f'Failed to load json file {root}/{file}, skipping')
                            logging.debug('Failure message:')
                            logging.debug(e, stack_info=True)

        if files:
            files = [os.path.abspath(file) for file in files]
            for file in files:
                if file.endswith(".json"):
                    tf_definitions, template_lines = parse_tf_plan(file)
                    if not tf_definitions:
                        continue
                    self.tf_definitions = tf_definitions
                    self.template_lines = template_lines
                    self.check_tf_definition(report, runner_filter)
                else:
                    logging.debug(f'Failed to load {file} as is not a .json file, skipping')

        report.add_parsing_errors(parsing_errors.keys())

        return report

    def check_tf_definition(self, report, runner_filter,
                            ):

        for full_file_path, definition in self.tf_definitions.items():
            scanned_file = f"/{os.path.relpath(full_file_path)}"
            logging.debug(f"Scanning file: {scanned_file}")
            for block_type in definition.keys():
                if block_type in self.block_type_registries.keys():
                    self.run_block(definition[block_type], full_file_path, report, scanned_file,
                                   block_type, runner_filter)

    def run_block(self, entities, full_file_path, report, scanned_file, block_type,
                  runner_filter=None):
        registry = self.block_type_registries[block_type]
        if registry:
            for entity in entities:
                entity_evaluations = None
                context_parser = parser_registry.context_parsers[block_type]
                definition_path = context_parser.get_entity_context_path(entity)
                entity_id = ".".join(definition_path)
                # Entity can exist only once per dir, for file as well
                entity_context = self.get_entity_context(definition_path, full_file_path)
                entity_lines_range = [entity_context.get('start_line'), entity_context.get('end_line')]
                entity_code_lines = entity_context.get('code_lines')
                results = registry.scan(scanned_file, entity, [], runner_filter)
                for check, check_result in results.items():
                    record = Record(check_id=check.id, check_name=check.name, check_result=check_result,
                                    code_block=entity_code_lines, file_path=scanned_file,
                                    file_line_range=entity_lines_range,
                                    resource=entity_id, evaluations=entity_evaluations,
                                    check_class=check.__class__.__module__, file_abs_path=full_file_path)
                    report.add_record(record=record)

    def get_entity_context(self, definition_path, full_file_path):
        entity_context = {}
        for resource in self.tf_definitions[full_file_path]['resource']:
            resource_type = definition_path[0]
            if resource_type in resource.keys():
                resource_name = definition_path[1]
                if resource_name in resource[resource_type].keys():
                    resource_defintion = resource[resource_type][resource_name]
                    entity_context['start_line'] = resource_defintion['start_line'][0]
                    entity_context['end_line'] = resource_defintion['end_line'][0]
                    entity_context['code_lines'] = self.template_lines[entity_context['start_line']:entity_context['end_line']]
                    return entity_context
        return entity_context



import re
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.provider.base_check import BaseProviderCheck
from checkov.common.models.consts import access_key_pattern, secret_key_pattern


class AWSCredentials(BaseProviderCheck):

    def __init__(self):
        name = "Ensure no hard coded AWS access key and secret key exists in provider"
        id = "CKV_AWS_41"
        supported_provider = ['aws']
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_provider=supported_provider)

    def scan_provider_conf(self, conf):
        """
        see: https://www.terraform.io/docs/providers/aws/index.html#static-credentials
        """
        if self.secret_found(conf, "access_key", access_key_pattern):
            return CheckResult.FAILED
        if self.secret_found(conf, "secret_key", secret_key_pattern):
            return CheckResult.FAILED
        return CheckResult.PASSED

    @staticmethod
    def secret_found(conf, field, pattern):
        if field in conf.keys():
            value = conf[field][0]
            if re.match(pattern, value) is not None:
                return True
        return False


check = AWSCredentials()



from networkx.classes.digraph import DiGraph

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.terraform.checks_infra.solvers.connections_solvers.complex_connection_solver import ComplexConnectionSolver
from checkov.terraform.graph_builder.graph_components.attribute_names import CustomAttributes


class AndConnectionSolver(ComplexConnectionSolver):
    operator = 'and'

    def __init__(self, queries, operator):
        super().__init__(queries, operator)

    def run(self, graph_connector: DiGraph):
        attribute_queries = [sub_query for sub_query in self.queries if
                             sub_query.solver_type in [SolverType.ATTRIBUTE, SolverType.COMPLEX]]
        passed_attributes, failed_attributes = [], []
        for attribute_query in attribute_queries:
            passed_query, failed_query = attribute_query.run(graph_connector)
            passed_attributes.extend(passed_query)
            failed_attributes.extend(failed_query)
            passed_attributes = [p for p in passed_attributes if
                                 p[CustomAttributes.ID] not in [f[CustomAttributes.ID] for f in failed_attributes]]

        passed, failed = passed_attributes, failed_attributes
        connection_queries = [sub_query for sub_query in self.queries if
                              sub_query.solver_type in [SolverType.CONNECTION, SolverType.COMPLEX_CONNECTION]]
        passed_connections, failed_connections = [], []
        for connection_query in connection_queries:
            passed_query, failed_query = connection_query.run(graph_connector)
            passed_query = [vertex for vertex in passed_query if
                                 vertex[CustomAttributes.ID] in [p[CustomAttributes.ID] for p in passed_attributes]]
            failed_query = [vertex for vertex in failed_connections if
                                 vertex[CustomAttributes.ID] not in [f[CustomAttributes.ID] for f in failed_connections]]
            passed_connections.extend(passed_query)
            failed_connections.extend(failed_query)
        passed.extend(passed_connections)
        failed.extend(failed_connections)

        return self.filter_results(passed, failed)
